"""
Location bookmarker script for Source engine games using Mattie's EventScripts

If an Auth provider script is loaded, the permissions of different commands
can be controlled via several capability names:

save_location
  * Allows people to use !bm command to bookmark their location
  * Allows people to use !dbm command to delete their saved bookmark
  * Fallback level: IDENTIFIED

teleport_own_location
  * Allows people to use !go command to teleport to bookmarked location
  * Fallback level: IDENTIFIED

teleport_any_location
  * Allows people to use !go2 command to teleport to location bookmarked by
    another player
  * Fallback level: IDENTIFIED

teleport_other_people
  * Allows people to use !here and !allto commands to teleport other players
  * Fallback level: ADMIN

teleport_entities
  * Allows people to use !move command to teleport a movable entity
  * Fallback level: POWERUSER
"""
import math

import vecmath
import es
import services

try:
    import cPickle as pickle
except ImportError:
    import pickle

info = es.AddonInfo()
info.name = "Location bookmarker"
info.basename = "loc_bookmarker"
info.author = "GODJonez"
info.version = "1.3"
info.description = "Allows bookmarking of locations for quick teleportation"

syn_events = None
mapname = es.ServerVar('eventscripts_currentmap')
savepath = es.getAddonPath(info.basename) + '/saved_locations/'
gamename = str(es.ServerVar('eventscripts_gamedir')).replace('\\', '/').rpartition('/')[2].lower()
if 'syn' in gamename:
    try:
        import syn_events
    except ImportError:
        pass

if not services.isRegistered('auth'):
    auth = lambda userid, permission: True
else:
    auth_service = services.use('auth')
    auth_service.registerCapability('save_location', auth_service.IDENTIFIED)
    auth_service.registerCapability('teleport_own_location', auth_service.IDENTIFIED)
    auth_service.registerCapability('teleport_any_location', auth_service.IDENTIFIED)
    auth_service.registerCapability('teleport_other_people', auth_service.ADMIN)
    auth_service.registerCapability('teleport_entities', auth_service.POWERUSER)
    auth = auth_service.isUseridAuthorized

playerlocations = {}

def uniqueid(uid):
    steamid = es.getplayersteamid(uid)
    if steamid in ('STEAM_ID_PENDING', 'BOT'):
        return 'userid_%s' % uid
    return steamid

def load():
    if syn_events:
        syn_events.register('syn_player_death', syn_player_death)
    users = es.getUseridList()
    for uid in users:
        playerlocations[uniqueid(uid)] = {}
    if str(mapname):
        load_locations(str(mapname))

def unload():
    if syn_events:
        syn_events.unregister('syn_player_death', syn_player_death)

def es_map_start(ev):
    playerlocations.clear()
    load_locations(ev['mapname'])

def player_death(ev):
    if not syn_events:
        syn_player_death(ev)

def syn_player_death(ev):
    uid = ev['userid']
    if uid and int(uid):
        saveLoc(int(uid), '0')

def saveLoc(uid, index):
    unid = uniqueid(uid)
    playerlocations[unid][index] = vecmath.Vector(es.getplayerprop(uid, "CBaseEntity.m_vecOrigin"))
    if unid != 'userid_%s' % uid:
       save_locations(str(mapname))

def delLoc(uid, index):
    unid = uniqueid(uid)
    del playerlocations[unid][index]
    if unid != 'userid_%s' % uid:
       save_locations(str(mapname))

def load_locations(mapname):
    fn = savepath + mapname
    try:
        fh = open(fn, 'rb')
    except IOError, error:
        es.dbgmsg(1, 'No saved bookmarks for map '+mapname+', '+str(error))
        return
    stored = pickle.load(fh)
    fh.close()
    playerlocations.update(stored)

def save_locations(mapname):
    fn = savepath + mapname
    fh = open(fn, 'wb')
    valids = [
        (unid, locations)
        for (unid, locations) in playerlocations.iteritems()
        if not unid.startswith('userid')
    ]
    pickle.dump(valids, fh, 2)
    fh.close()
    
def player_activate(ev):
    uid = int(ev['userid'])
    unid = uniqueid(uid)
    if unid not in playerlocations:
        playerlocations[unid] = {}
    
def es_player_chat(ev):
    uid = int(ev['userid'])
    unid = uniqueid(uid)
    text = ev['text'].split(' ',1)
    if not text:
        return
    command = text[0]
    if len(text) > 1:
        index = text[1]
    else:
        index = '0'
    if command == '!go':
        if auth(uid, 'teleport_own_location'):
            if index in playerlocations[unid]:
                es.server.queuecmd('es_xsetpos %s %s'%(uid, playerlocations[unid][index].getstr(' ')))
            else:
                es.tell(uid, 'You do not have that location bookmarked')
        else:
            es.tell(uid, 'You do not have permission to use this command')
    elif command == '!go2':
        if auth(uid, 'teleport_any_location'):
            for xid in playerlocations:
                if index in playerlocations[xid]:
                    es.server.queuecmd('es_xsetpos %s %s'%(uid, playerlocations[xid][index].getstr(' ')))
                    return
        else:
            es.tell(uid, 'You do not have permission to use this command')
    elif command == '!bm':
        if auth(uid, 'save_location'):
            saveLoc(uid, index)
        else:
            es.tell(uid, 'You do not have permission to use this command')
    elif command == '!dbm':
        if auth(uid, 'save_location'):
            if index in playerlocations[unid]:
                delLoc(uid, index)
            else:
                es.tell(uid, 'You do not have that location bookmarked')
        else:
            es.tell(uid, 'You do not have permission to use this command')
    elif command == '!move':
        if auth(uid, 'teleport_entities'):
            if index in playerlocations[uid]:
                loc = playerlocations[uid][index]
            else:
                loc = None
                for xid in playerlocations:
                    if index in playerlocations[xid]:
                        loc = playerlocations[xid][index]
                        break
                if loc is None:
                    es.tell(uid, 'No such bookmark found')
                    return
            es.server.queuecmd('es_xfire %s !picker addoutput "origin %s"'%(uid, loc.getstr(' ')))
        else:
            es.tell(uid, 'You do not have permission to use this command')
    elif command in ('!here', '!allto'):
        if auth(uid, 'teleport_other_people'):
            if command == '!here':
                loc = vecmath.Vector(es.getplayerprop(uid, "CBaseEntity.m_vecOrigin"))
            elif index in playerlocations[unid]:
                loc = playerlocations[unid][index]
            else:
                loc = None
                for xid in playerlocations:
                    if index in playerlocations[xid]:
                        loc = playerlocations[xid][index]
                        break
                if loc is None:
                    es.tell(uid, 'No such bookmark found')
                    return
            players = set(es.getUseridList())
            players.remove(uid)
            users = len(players)
            # teleport self
            es.server.queuecmd('es_xsetpos %s %s'%(uid, loc.getstr(' ')))
            # teleport others
            for idx, userid in enumerate(players):
                angle = 2*math.pi*idx/users
                tloc = loc + 50*vecmath.Vector(math.cos(angle), math.sin(angle), 0)
                es.server.queuecmd('es_xsetpos %s %s'%(userid, tloc.getstr(' ')))
        else:
            es.tell(uid, 'You do not have permission to use this command')
